# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/exporters/manuskript/plainTextSettings_ui.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_exporterSettings(object):
    def setupUi(self, exporterSettings):
        exporterSettings.setObjectName("exporterSettings")
        exporterSettings.resize(511, 626)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(exporterSettings)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.toolBox = QtWidgets.QToolBox(exporterSettings)
        self.toolBox.setObjectName("toolBox")
        self.content = QtWidgets.QWidget()
        self.content.setGeometry(QtCore.QRect(0, 0, 349, 842))
        self.content.setObjectName("content")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.content)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label = QtWidgets.QLabel(self.content)
        self.label.setObjectName("label")
        self.verticalLayout_5.addWidget(self.label)
        self.tblContent = QtWidgets.QTableWidget(self.content)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tblContent.sizePolicy().hasHeightForWidth())
        self.tblContent.setSizePolicy(sizePolicy)
        self.tblContent.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblContent.setAlternatingRowColors(True)
        self.tblContent.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblContent.setObjectName("tblContent")
        self.tblContent.setColumnCount(3)
        self.tblContent.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblContent.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblContent.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblContent.setHorizontalHeaderItem(2, item)
        self.tblContent.horizontalHeader().setMinimumSectionSize(75)
        self.tblContent.verticalHeader().setVisible(False)
        self.verticalLayout_5.addWidget(self.tblContent)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.chkContentMore = QtWidgets.QCheckBox(self.content)
        self.chkContentMore.setObjectName("chkContentMore")
        self.horizontalLayout_8.addWidget(self.chkContentMore)
        self.verticalLayout_5.addLayout(self.horizontalLayout_8)
        self.grpContentFilters = collapsibleGroupBox2(self.content)
        self.grpContentFilters.setCheckable(True)
        self.grpContentFilters.setObjectName("grpContentFilters")
        self.formLayout_2 = QtWidgets.QFormLayout(self.grpContentFilters)
        self.formLayout_2.setRowWrapPolicy(QtWidgets.QFormLayout.WrapLongRows)
        self.formLayout_2.setLabelAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_2 = QtWidgets.QLabel(self.grpContentFilters)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.label_2)
        self.chkContentIgnoreCompile = QtWidgets.QCheckBox(self.grpContentFilters)
        self.chkContentIgnoreCompile.setObjectName("chkContentIgnoreCompile")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.SpanningRole, self.chkContentIgnoreCompile)
        self.chkContentParent = QtWidgets.QCheckBox(self.grpContentFilters)
        self.chkContentParent.setObjectName("chkContentParent")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.chkContentParent)
        self.cmbContentParent = QtWidgets.QComboBox(self.grpContentFilters)
        self.cmbContentParent.setFrame(False)
        self.cmbContentParent.setObjectName("cmbContentParent")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.cmbContentParent)
        self.chkContentLabels = QtWidgets.QCheckBox(self.grpContentFilters)
        self.chkContentLabels.setObjectName("chkContentLabels")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.chkContentLabels)
        self.chkContentStatus = QtWidgets.QCheckBox(self.grpContentFilters)
        self.chkContentStatus.setObjectName("chkContentStatus")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.chkContentStatus)
        self.lstContentLabels = QtWidgets.QListWidget(self.grpContentFilters)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstContentLabels.sizePolicy().hasHeightForWidth())
        self.lstContentLabels.setSizePolicy(sizePolicy)
        self.lstContentLabels.setObjectName("lstContentLabels")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.lstContentLabels)
        self.lstContentStatus = QtWidgets.QListWidget(self.grpContentFilters)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstContentStatus.sizePolicy().hasHeightForWidth())
        self.lstContentStatus.setSizePolicy(sizePolicy)
        self.lstContentStatus.setObjectName("lstContentStatus")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.lstContentStatus)
        self.verticalLayout_5.addWidget(self.grpContentFilters)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem1)
        self.toolBox.addItem(self.content, "")
        self.separations = QtWidgets.QWidget()
        self.separations.setGeometry(QtCore.QRect(0, 0, 173, 336))
        self.separations.setObjectName("separations")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.separations)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_3 = QtWidgets.QLabel(self.separations)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_8.addWidget(self.label_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.pushButton_4 = QtWidgets.QPushButton(self.separations)
        self.pushButton_4.setEnabled(True)
        self.pushButton_4.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_4.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_4.setText("")
        icon = QtGui.QIcon.fromTheme("folder")
        self.pushButton_4.setIcon(icon)
        self.pushButton_4.setFlat(True)
        self.pushButton_4.setObjectName("pushButton_4")
        self.verticalLayout.addWidget(self.pushButton_4)
        self.line = QtWidgets.QFrame(self.separations)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.pushButton = QtWidgets.QPushButton(self.separations)
        self.pushButton.setEnabled(True)
        self.pushButton.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton.setText("")
        icon = QtGui.QIcon.fromTheme("folder")
        self.pushButton.setIcon(icon)
        self.pushButton.setFlat(True)
        self.pushButton.setObjectName("pushButton")
        self.verticalLayout.addWidget(self.pushButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.cmbSepFF = QtWidgets.QComboBox(self.separations)
        self.cmbSepFF.setObjectName("cmbSepFF")
        self.cmbSepFF.addItem("")
        self.cmbSepFF.addItem("")
        self.horizontalLayout.addWidget(self.cmbSepFF)
        self.txtSepFF = QtWidgets.QLineEdit(self.separations)
        self.txtSepFF.setEnabled(False)
        self.txtSepFF.setText("\\n")
        self.txtSepFF.setObjectName("txtSepFF")
        self.horizontalLayout.addWidget(self.txtSepFF)
        self.verticalLayout_8.addLayout(self.horizontalLayout)
        self.label_4 = QtWidgets.QLabel(self.separations)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_8.addWidget(self.label_4)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem3 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pushButton_5 = QtWidgets.QPushButton(self.separations)
        self.pushButton_5.setEnabled(True)
        self.pushButton_5.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_5.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_5.setText("")
        icon = QtGui.QIcon.fromTheme("text-x-generic")
        self.pushButton_5.setIcon(icon)
        self.pushButton_5.setFlat(True)
        self.pushButton_5.setObjectName("pushButton_5")
        self.verticalLayout_3.addWidget(self.pushButton_5)
        self.line_2 = QtWidgets.QFrame(self.separations)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_3.addWidget(self.line_2)
        self.pushButton_6 = QtWidgets.QPushButton(self.separations)
        self.pushButton_6.setEnabled(True)
        self.pushButton_6.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_6.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_6.setText("")
        icon = QtGui.QIcon.fromTheme("text-x-generic")
        self.pushButton_6.setIcon(icon)
        self.pushButton_6.setFlat(True)
        self.pushButton_6.setObjectName("pushButton_6")
        self.verticalLayout_3.addWidget(self.pushButton_6)
        self.horizontalLayout_2.addLayout(self.verticalLayout_3)
        self.cmbSepTT = QtWidgets.QComboBox(self.separations)
        self.cmbSepTT.setObjectName("cmbSepTT")
        self.cmbSepTT.addItem("")
        self.cmbSepTT.addItem("")
        self.horizontalLayout_2.addWidget(self.cmbSepTT)
        self.txtSepTT = QtWidgets.QLineEdit(self.separations)
        self.txtSepTT.setEnabled(False)
        self.txtSepTT.setText("\\n")
        self.txtSepTT.setObjectName("txtSepTT")
        self.horizontalLayout_2.addWidget(self.txtSepTT)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.label_6 = QtWidgets.QLabel(self.separations)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_8.addWidget(self.label_6)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem4 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.pushButton_7 = QtWidgets.QPushButton(self.separations)
        self.pushButton_7.setEnabled(True)
        self.pushButton_7.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_7.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_7.setText("")
        icon = QtGui.QIcon.fromTheme("folder")
        self.pushButton_7.setIcon(icon)
        self.pushButton_7.setFlat(True)
        self.pushButton_7.setObjectName("pushButton_7")
        self.verticalLayout_4.addWidget(self.pushButton_7)
        self.line_3 = QtWidgets.QFrame(self.separations)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_4.addWidget(self.line_3)
        self.pushButton_8 = QtWidgets.QPushButton(self.separations)
        self.pushButton_8.setEnabled(True)
        self.pushButton_8.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_8.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_8.setText("")
        icon = QtGui.QIcon.fromTheme("text-x-generic")
        self.pushButton_8.setIcon(icon)
        self.pushButton_8.setFlat(True)
        self.pushButton_8.setObjectName("pushButton_8")
        self.verticalLayout_4.addWidget(self.pushButton_8)
        self.horizontalLayout_3.addLayout(self.verticalLayout_4)
        self.cmbSepFT = QtWidgets.QComboBox(self.separations)
        self.cmbSepFT.setObjectName("cmbSepFT")
        self.cmbSepFT.addItem("")
        self.cmbSepFT.addItem("")
        self.horizontalLayout_3.addWidget(self.cmbSepFT)
        self.txtSepFT = QtWidgets.QLineEdit(self.separations)
        self.txtSepFT.setEnabled(False)
        self.txtSepFT.setText("\\n")
        self.txtSepFT.setObjectName("txtSepFT")
        self.horizontalLayout_3.addWidget(self.txtSepFT)
        self.verticalLayout_8.addLayout(self.horizontalLayout_3)
        self.label_5 = QtWidgets.QLabel(self.separations)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_8.addWidget(self.label_5)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem5 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem5)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.pushButton_9 = QtWidgets.QPushButton(self.separations)
        self.pushButton_9.setEnabled(True)
        self.pushButton_9.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_9.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_9.setText("")
        icon = QtGui.QIcon.fromTheme("text-x-generic")
        self.pushButton_9.setIcon(icon)
        self.pushButton_9.setFlat(True)
        self.pushButton_9.setObjectName("pushButton_9")
        self.verticalLayout_7.addWidget(self.pushButton_9)
        self.line_4 = QtWidgets.QFrame(self.separations)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_7.addWidget(self.line_4)
        self.pushButton_10 = QtWidgets.QPushButton(self.separations)
        self.pushButton_10.setEnabled(True)
        self.pushButton_10.setMinimumSize(QtCore.QSize(16, 16))
        self.pushButton_10.setMaximumSize(QtCore.QSize(16, 16))
        self.pushButton_10.setText("")
        icon = QtGui.QIcon.fromTheme("folder")
        self.pushButton_10.setIcon(icon)
        self.pushButton_10.setFlat(True)
        self.pushButton_10.setObjectName("pushButton_10")
        self.verticalLayout_7.addWidget(self.pushButton_10)
        self.horizontalLayout_4.addLayout(self.verticalLayout_7)
        self.cmbSepTF = QtWidgets.QComboBox(self.separations)
        self.cmbSepTF.setObjectName("cmbSepTF")
        self.cmbSepTF.addItem("")
        self.cmbSepTF.addItem("")
        self.horizontalLayout_4.addWidget(self.cmbSepTF)
        self.txtSepTF = QtWidgets.QLineEdit(self.separations)
        self.txtSepTF.setEnabled(False)
        self.txtSepTF.setText("\\n")
        self.txtSepTF.setObjectName("txtSepTF")
        self.horizontalLayout_4.addWidget(self.txtSepTF)
        self.verticalLayout_8.addLayout(self.horizontalLayout_4)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem6)
        self.toolBox.addItem(self.separations, "")
        self.transformations = QtWidgets.QWidget()
        self.transformations.setGeometry(QtCore.QRect(0, 0, 511, 522))
        self.transformations.setStyleSheet("QGroupBox{font-weight:bold;}")
        self.transformations.setObjectName("transformations")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.transformations)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.grpTransTypo = collapsibleGroupBox2(self.transformations)
        self.grpTransTypo.setStyleSheet("")
        self.grpTransTypo.setObjectName("grpTransTypo")
        self.formLayout = QtWidgets.QFormLayout(self.grpTransTypo)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldsStayAtSizeHint)
        self.formLayout.setRowWrapPolicy(QtWidgets.QFormLayout.WrapLongRows)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.chkTransEllipse = QtWidgets.QCheckBox(self.grpTransTypo)
        self.chkTransEllipse.setObjectName("chkTransEllipse")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.chkTransEllipse)
        self.chkTransDash = QtWidgets.QCheckBox(self.grpTransTypo)
        self.chkTransDash.setObjectName("chkTransDash")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.chkTransDash)
        self.chkTransDoubleQuotes = QtWidgets.QCheckBox(self.grpTransTypo)
        self.chkTransDoubleQuotes.setObjectName("chkTransDoubleQuotes")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.chkTransDoubleQuotes)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(12)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.cmbTransDoubleQuotes = QtWidgets.QComboBox(self.grpTransTypo)
        self.cmbTransDoubleQuotes.setFrame(False)
        self.cmbTransDoubleQuotes.setObjectName("cmbTransDoubleQuotes")
        self.horizontalLayout_5.addWidget(self.cmbTransDoubleQuotes)
        self.txtTransDoubleQuotesA = QtWidgets.QLineEdit(self.grpTransTypo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtTransDoubleQuotesA.sizePolicy().hasHeightForWidth())
        self.txtTransDoubleQuotesA.setSizePolicy(sizePolicy)
        self.txtTransDoubleQuotesA.setMinimumSize(QtCore.QSize(40, 0))
        self.txtTransDoubleQuotesA.setMaximumSize(QtCore.QSize(40, 16777215))
        self.txtTransDoubleQuotesA.setStyleSheet("border-radius: 6px;")
        self.txtTransDoubleQuotesA.setText("")
        self.txtTransDoubleQuotesA.setFrame(False)
        self.txtTransDoubleQuotesA.setAlignment(QtCore.Qt.AlignCenter)
        self.txtTransDoubleQuotesA.setObjectName("txtTransDoubleQuotesA")
        self.horizontalLayout_5.addWidget(self.txtTransDoubleQuotesA)
        self.lblTransDoubleQuotes = QtWidgets.QLabel(self.grpTransTypo)
        self.lblTransDoubleQuotes.setText("…")
        self.lblTransDoubleQuotes.setObjectName("lblTransDoubleQuotes")
        self.horizontalLayout_5.addWidget(self.lblTransDoubleQuotes)
        self.txtTransDoubleQuotesB = QtWidgets.QLineEdit(self.grpTransTypo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtTransDoubleQuotesB.sizePolicy().hasHeightForWidth())
        self.txtTransDoubleQuotesB.setSizePolicy(sizePolicy)
        self.txtTransDoubleQuotesB.setMinimumSize(QtCore.QSize(40, 0))
        self.txtTransDoubleQuotesB.setMaximumSize(QtCore.QSize(40, 16777215))
        self.txtTransDoubleQuotesB.setStyleSheet("border-radius: 6px;")
        self.txtTransDoubleQuotesB.setText("")
        self.txtTransDoubleQuotesB.setFrame(False)
        self.txtTransDoubleQuotesB.setAlignment(QtCore.Qt.AlignCenter)
        self.txtTransDoubleQuotesB.setObjectName("txtTransDoubleQuotesB")
        self.horizontalLayout_5.addWidget(self.txtTransDoubleQuotesB)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem7)
        self.formLayout.setLayout(2, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_5)
        self.chkTransSingleQuote = QtWidgets.QCheckBox(self.grpTransTypo)
        self.chkTransSingleQuote.setObjectName("chkTransSingleQuote")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.chkTransSingleQuote)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setSpacing(12)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.cmbTransSingleQuote = QtWidgets.QComboBox(self.grpTransTypo)
        self.cmbTransSingleQuote.setFrame(False)
        self.cmbTransSingleQuote.setObjectName("cmbTransSingleQuote")
        self.horizontalLayout_6.addWidget(self.cmbTransSingleQuote)
        self.txtTransSingleQuoteA = QtWidgets.QLineEdit(self.grpTransTypo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtTransSingleQuoteA.sizePolicy().hasHeightForWidth())
        self.txtTransSingleQuoteA.setSizePolicy(sizePolicy)
        self.txtTransSingleQuoteA.setMinimumSize(QtCore.QSize(40, 0))
        self.txtTransSingleQuoteA.setMaximumSize(QtCore.QSize(40, 16777215))
        self.txtTransSingleQuoteA.setStyleSheet("border-radius: 6px;")
        self.txtTransSingleQuoteA.setText("")
        self.txtTransSingleQuoteA.setFrame(False)
        self.txtTransSingleQuoteA.setAlignment(QtCore.Qt.AlignCenter)
        self.txtTransSingleQuoteA.setObjectName("txtTransSingleQuoteA")
        self.horizontalLayout_6.addWidget(self.txtTransSingleQuoteA)
        self.lblTransSingleQuote = QtWidgets.QLabel(self.grpTransTypo)
        self.lblTransSingleQuote.setText("…")
        self.lblTransSingleQuote.setObjectName("lblTransSingleQuote")
        self.horizontalLayout_6.addWidget(self.lblTransSingleQuote)
        self.txtTransSingleQuoteB = QtWidgets.QLineEdit(self.grpTransTypo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtTransSingleQuoteB.sizePolicy().hasHeightForWidth())
        self.txtTransSingleQuoteB.setSizePolicy(sizePolicy)
        self.txtTransSingleQuoteB.setMinimumSize(QtCore.QSize(40, 0))
        self.txtTransSingleQuoteB.setMaximumSize(QtCore.QSize(40, 16777215))
        self.txtTransSingleQuoteB.setStyleSheet("border-radius: 6px;")
        self.txtTransSingleQuoteB.setText("")
        self.txtTransSingleQuoteB.setFrame(False)
        self.txtTransSingleQuoteB.setAlignment(QtCore.Qt.AlignCenter)
        self.txtTransSingleQuoteB.setObjectName("txtTransSingleQuoteB")
        self.horizontalLayout_6.addWidget(self.txtTransSingleQuoteB)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem8)
        self.formLayout.setLayout(3, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_6)
        self.chkTransSpaces = QtWidgets.QCheckBox(self.grpTransTypo)
        self.chkTransSpaces.setObjectName("chkTransSpaces")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.chkTransSpaces)
        self.verticalLayout_6.addWidget(self.grpTransTypo)
        self.grpTransCustom = collapsibleGroupBox2(self.transformations)
        self.grpTransCustom.setObjectName("grpTransCustom")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.grpTransCustom)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tblReplacements = QtWidgets.QTableWidget(self.grpTransCustom)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tblReplacements.sizePolicy().hasHeightForWidth())
        self.tblReplacements.setSizePolicy(sizePolicy)
        self.tblReplacements.setAlternatingRowColors(True)
        self.tblReplacements.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblReplacements.setObjectName("tblReplacements")
        self.tblReplacements.setColumnCount(4)
        self.tblReplacements.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblReplacements.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblReplacements.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblReplacements.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblReplacements.setHorizontalHeaderItem(3, item)
        self.verticalLayout_9.addWidget(self.tblReplacements)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem9)
        self.btnTransAdd = QtWidgets.QPushButton(self.grpTransCustom)
        self.btnTransAdd.setText("")
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnTransAdd.setIcon(icon)
        self.btnTransAdd.setFlat(True)
        self.btnTransAdd.setObjectName("btnTransAdd")
        self.horizontalLayout_7.addWidget(self.btnTransAdd)
        self.btnTransRemove = QtWidgets.QPushButton(self.grpTransCustom)
        self.btnTransRemove.setText("")
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnTransRemove.setIcon(icon)
        self.btnTransRemove.setFlat(True)
        self.btnTransRemove.setObjectName("btnTransRemove")
        self.horizontalLayout_7.addWidget(self.btnTransRemove)
        self.verticalLayout_9.addLayout(self.horizontalLayout_7)
        self.verticalLayout_6.addWidget(self.grpTransCustom)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem10)
        self.toolBox.addItem(self.transformations, "")
        self.preview = QtWidgets.QWidget()
        self.preview.setGeometry(QtCore.QRect(0, 0, 369, 130))
        self.preview.setStyleSheet("QGroupBox{font-weight:bold;}")
        self.preview.setObjectName("preview")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.preview)
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.groupBox = QtWidgets.QGroupBox(self.preview)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_3 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_3.setObjectName("formLayout_3")
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.cmbPreviewFont = QtWidgets.QFontComboBox(self.groupBox)
        self.cmbPreviewFont.setObjectName("cmbPreviewFont")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbPreviewFont)
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.spnPreviewSize = QtWidgets.QSpinBox(self.groupBox)
        self.spnPreviewSize.setObjectName("spnPreviewSize")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.spnPreviewSize)
        self.verticalLayout_11.addWidget(self.groupBox)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_11.addItem(spacerItem11)
        self.toolBox.addItem(self.preview, "")
        self.verticalLayout_2.addWidget(self.toolBox)

        self.retranslateUi(exporterSettings)
        self.toolBox.setCurrentIndex(2)
        self.toolBox.layout().setSpacing(0)
        QtCore.QMetaObject.connectSlotsByName(exporterSettings)

    def retranslateUi(self, exporterSettings):
        _translate = QtCore.QCoreApplication.translate
        exporterSettings.setWindowTitle(_translate("exporterSettings", "Form"))
        self.label.setText(_translate("exporterSettings", "Decide here what will be included in the final export."))
        item = self.tblContent.horizontalHeaderItem(0)
        item.setText(_translate("exporterSettings", "Type"))
        item = self.tblContent.horizontalHeaderItem(1)
        item.setText(_translate("exporterSettings", "Title"))
        item = self.tblContent.horizontalHeaderItem(2)
        item.setText(_translate("exporterSettings", "Text"))
        self.chkContentMore.setText(_translate("exporterSettings", "I need more granularity"))
        self.grpContentFilters.setTitle(_translate("exporterSettings", "Fi&lters"))
        self.label_2.setText(_translate("exporterSettings", "<html><head/><body><p>Filters what items will be included in the final export.<br/><span style=\" color:#773333;\">(Not fully implemented yet.)</span></p></body></html>"))
        self.chkContentIgnoreCompile.setText(_translate("exporterSettings", "Ignore compile status (include all items)"))
        self.chkContentParent.setText(_translate("exporterSettings", "Subitems of:"))
        self.chkContentLabels.setText(_translate("exporterSettings", "Labels"))
        self.chkContentStatus.setText(_translate("exporterSettings", "Status"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.content), _translate("exporterSettings", "Content"))
        self.label_3.setText(_translate("exporterSettings", "Between folders:"))
        self.cmbSepFF.setItemText(0, _translate("exporterSettings", "Empty line"))
        self.cmbSepFF.setItemText(1, _translate("exporterSettings", "Custom"))
        self.label_4.setText(_translate("exporterSettings", "Between texts:"))
        self.cmbSepTT.setItemText(0, _translate("exporterSettings", "Empty line"))
        self.cmbSepTT.setItemText(1, _translate("exporterSettings", "Custom"))
        self.label_6.setText(_translate("exporterSettings", "Between folder and text:"))
        self.cmbSepFT.setItemText(0, _translate("exporterSettings", "Empty line"))
        self.cmbSepFT.setItemText(1, _translate("exporterSettings", "Custom"))
        self.label_5.setText(_translate("exporterSettings", "Between text and folder:"))
        self.cmbSepTF.setItemText(0, _translate("exporterSettings", "Empty line"))
        self.cmbSepTF.setItemText(1, _translate("exporterSettings", "Custom"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.separations), _translate("exporterSettings", "Separations"))
        self.grpTransTypo.setTitle(_translate("exporterSettings", "Typographic replacements:"))
        self.chkTransEllipse.setText(_translate("exporterSettings", "Replace ... with …"))
        self.chkTransDash.setText(_translate("exporterSettings", "Replace --- with —"))
        self.chkTransDoubleQuotes.setText(_translate("exporterSettings", "Replace double quotes (\") with:"))
        self.chkTransSingleQuote.setText(_translate("exporterSettings", "Replace single quotes (\') with:"))
        self.chkTransSpaces.setText(_translate("exporterSettings", "Remove multiple spaces"))
        self.grpTransCustom.setTitle(_translate("exporterSettings", "Custom replacements:"))
        item = self.tblReplacements.horizontalHeaderItem(0)
        item.setText(_translate("exporterSettings", "Enabled"))
        item = self.tblReplacements.horizontalHeaderItem(1)
        item.setText(_translate("exporterSettings", "Replace"))
        item = self.tblReplacements.horizontalHeaderItem(2)
        item.setText(_translate("exporterSettings", "With"))
        item = self.tblReplacements.horizontalHeaderItem(3)
        item.setText(_translate("exporterSettings", "RegExp"))
        item.setWhatsThis(_translate("exporterSettings", "If checked, uses regular expression for replacement. If unchecked, replaced as plain text."))
        self.toolBox.setItemText(self.toolBox.indexOf(self.transformations), _translate("exporterSettings", "Transformations"))
        self.groupBox.setTitle(_translate("exporterSettings", "Font"))
        self.label_7.setText(_translate("exporterSettings", "Font:"))
        self.label_8.setText(_translate("exporterSettings", "Font size:"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.preview), _translate("exporterSettings", "Preview"))

from manuskript.ui.collapsibleGroupBox2 import collapsibleGroupBox2
